@@ -69,6 +69,7 @@ class AlitaClient:
         self.configurations_url = f'{self.base_url}{self.api_path}/integrations/integrations/default/{self.project_id}?section=configurations&unsecret=true'
         self.ai_section_url = f'{self.base_url}{self.api_path}/integrations/integrations/default/{self.project_id}?section=ai'
         self.configurations: list = configurations or []
+        self.model_timeout = kwargs.get('model_timeout', 120)
 
     def get_mcp_toolkits(self):
         if user_id := self._get_real_user_id():
@@ -85,7 +86,12 @@ class AlitaClient:
             # This loop iterates over each key-value pair in the arguments dictionary,
             # and if a value is a Pydantic object, it replaces it with its dictionary representation using .dict().
             for arg_name, arg_value in params.get('params', {}).get('arguments', {}).items():
-                if hasattr(arg_value, "dict") and callable(arg_value.dict):
+                if isinstance(arg_value, list):
+                    params['params']['arguments'][arg_name] = [
+                        item.dict() if hasattr(item, "dict") and callable(item.dict) else item
+                        for item in arg_value
+                    ]
+                elif hasattr(arg_value, "dict") and callable(arg_value.dict):
                     params['params']['arguments'][arg_name] = arg_value.dict()
             #
             response = requests.post(url, headers=self.headers, json=params, verify=False)
@@ -179,6 +185,7 @@ class AlitaClient:
             model=embedding_model,
             api_key=self.auth_token,
             openai_organization=str(self.project_id),
+            request_timeout=self.model_timeout
         )
 
     def get_llm(self, model_name: str, model_config: dict) -> ChatOpenAI:
@@ -349,7 +356,8 @@ class AlitaClient:
         return self._process_requst(resp)
 
     def list_artifacts(self, bucket_name: str):
-        url = f'{self.artifacts_url}/{bucket_name}'
+        # Ensure bucket name is lowercase as required by the API
+        url = f'{self.artifacts_url}/{bucket_name.lower()}'
         data = requests.get(url, headers=self.headers, verify=False)
         return self._process_requst(data)
 
