@@ -17,6 +17,38 @@ from ..utils.available_tools_decorator import extend_with_parent_available_tools
 from ..utils.content_parser import load_content_from_bytes
 
 GLOBAL_LIMIT = 10000
+GLOBAL_RETAIN = ['id', 'name', 'type', 'document', 'children']
+GLOBAL_REMOVE = []
+GLOBAL_DEPTH_START = 4
+GLOBAL_DEPTH_END = 6
+EXTRA_PARAMS = (
+    Optional[Dict[str, Union[str, int, List, None]]],
+    Field(
+        description=(
+            "Additional parameters for customizing response processing:\n"
+            "- `limit`: Maximum size of the output in characters.\n"
+            "- `regexp`: Regex pattern to filter or clean the output.\n"
+            "- `fields_retain`: List of field names to always keep in the output, on levels starting from `depth_start`.\n"
+            "- `fields_remove`: List of field names to exclude from the output, unless also present in `fields_retain`.\n"
+            "- `depth_start`: The depth in the object hierarchy at which field filtering begins (fields are retained or removed).\n"
+            "- `depth_end`: The depth at which all fields are ignored and recursion stops.\n"
+            "Use these parameters to control the granularity and size of the returned data, especially for large or deeply nested objects."
+        ),
+        default={
+            "limit": GLOBAL_LIMIT, "regexp": None,
+            "fields_retain": GLOBAL_RETAIN, "fields_remove": GLOBAL_REMOVE,
+            "depth_start": GLOBAL_DEPTH_START, "depth_end": GLOBAL_DEPTH_END,
+        },
+        examples=[
+            {
+                "limit": "1000",
+                "regexp": r'("strokes"|"fills")\s*:\s*("[^"]*"|[^\s,}\[]+)\s*(?=,|\}|\n)',
+                "fields_retain": GLOBAL_RETAIN, "fields_remove": GLOBAL_REMOVE,
+                "depth_start": GLOBAL_DEPTH_START, "depth_end": GLOBAL_DEPTH_END,
+            }
+        ],
+    ),
+)
 
 
 class ArgsSchema(Enum):
@@ -36,19 +68,7 @@ class ArgsSchema(Enum):
                 examples=["8:6,1:7"],
             ),
         ),
-        extra_params=(
-            Optional[Dict[str, Union[str, int, None]]],
-            Field(
-                description="Additional parameters including limit and regex pattern to be removed from response",
-                default={"limit": GLOBAL_LIMIT, "regexp": None},
-                examples=[
-                    {
-                        "limit": "1000",
-                        "regexp": r'("strokes"|"fills")\s*:\s*("[^"]*"|[^\s,}\[]+)\s*(?=,|\}|\n)',
-                    }
-                ],
-            ),
-        ),
+        extra_params=EXTRA_PARAMS,
     )
     File = create_model(
         "FileNodes",
@@ -67,19 +87,7 @@ class ArgsSchema(Enum):
             Optional[str],
             Field(description="Sets version of file", default=None),
         ),
-        extra_params=(
-            Optional[Dict[str, Union[str, int, None]]],
-            Field(
-                description="Additional parameters including limit and regex pattern to be removed from response",
-                default={"limit": GLOBAL_LIMIT, "regexp": None},
-                examples=[
-                    {
-                        "limit": "1000",
-                        "regexp": r'("strokes"|"fills")\s*:\s*("[^"]*"|[^\s,}\[]+)\s*(?=,|\}|\n)',
-                    }
-                ],
-            ),
-        ),
+        extra_params=EXTRA_PARAMS,
     )
     FileKey = create_model(
         "FileKey",
@@ -90,19 +98,7 @@ class ArgsSchema(Enum):
                 examples=["Fp24FuzPwH0L74ODSrCnQo"],
             ),
         ),
-        extra_params=(
-            Optional[Dict[str, Union[str, int, None]]],
-            Field(
-                description="Additional parameters including limit and regex pattern to be removed from response",
-                default={"limit": GLOBAL_LIMIT, "regexp": None},
-                examples=[
-                    {
-                        "limit": "1000",
-                        "regexp": r'("strokes"|"fills")\s*:\s*("[^"]*"|[^\s,}\[]+)\s*(?=,|\}|\n)',
-                    }
-                ],
-            ),
-        ),
+        extra_params=EXTRA_PARAMS,
     )
     FileComment = create_model(
         "FileComment",
@@ -124,19 +120,7 @@ class ArgsSchema(Enum):
                 default=None,
             ),
         ),
-        extra_params=(
-            Optional[Dict[str, Union[str, int, None]]],
-            Field(
-                description="Additional parameters including limit and regex pattern to be removed from response",
-                default={"limit": GLOBAL_LIMIT, "regexp": None},
-                examples=[
-                    {
-                        "limit": "1000",
-                        "regexp": r'("strokes"|"fills")\s*:\s*("[^"]*"|[^\s,}\[]+)\s*(?=,|\}|\n)',
-                    }
-                ],
-            ),
-        ),
+        extra_params=EXTRA_PARAMS,
     )
     FileImages = create_model(
         "FileImages",
@@ -171,19 +155,7 @@ class ArgsSchema(Enum):
             Optional[str],
             Field(description="A specific version ID to use", default=None),
         ),
-        extra_params=(
-            Optional[Dict[str, Union[str, int, None]]],
-            Field(
-                description="Additional parameters including limit and regex pattern to be removed from response",
-                default={"limit": GLOBAL_LIMIT, "regexp": None},
-                examples=[
-                    {
-                        "limit": "1000",
-                        "regexp": r'("strokes"|"fills")\s*:\s*("[^"]*"|[^\s,}\[]+)\s*(?=,|\}|\n)',
-                    }
-                ],
-            ),
-        ),
+        extra_params=EXTRA_PARAMS,
     )
     TeamProjects = create_model(
         "TeamProjects",
@@ -194,19 +166,7 @@ class ArgsSchema(Enum):
                 examples=["1101853299713989222"],
             ),
         ),
-        extra_params=(
-            Optional[Dict[str, Union[str, int, None]]],
-            Field(
-                description="Additional parameters including limit and regex pattern to be removed from response",
-                default={"limit": GLOBAL_LIMIT, "regexp": None},
-                examples=[
-                    {
-                        "limit": "1000",
-                        "regexp": r'("strokes"|"fills")\s*:\s*("[^"]*"|[^\s,}\[]+)\s*(?=,|\}|\n)',
-                    }
-                ],
-            ),
-        ),
+        extra_params=EXTRA_PARAMS,
     )
     ProjectFiles = create_model(
         "ProjectFiles",
@@ -217,19 +177,7 @@ class ArgsSchema(Enum):
                 examples=["55391681"],
             ),
         ),
-        extra_params=(
-            Optional[Dict[str, Union[str, int, None]]],
-            Field(
-                description="Additional parameters including limit and regex pattern to be removed from response",
-                default={"limit": GLOBAL_LIMIT, "regexp": None},
-                examples=[
-                    {
-                        "limit": "1000",
-                        "regexp": r'("strokes"|"fills")\s*:\s*("[^"]*"|[^\s,}\[]+)\s*(?=,|\}|\n)',
-                    }
-                ],
-            ),
-        ),
+        extra_params=EXTRA_PARAMS,
     )
 
 
@@ -238,6 +186,10 @@ class FigmaApiWrapper(NonCodeIndexerToolkit):
     oauth2: Optional[SecretStr] = Field(default=None)
     global_limit: Optional[int] = Field(default=GLOBAL_LIMIT)
     global_regexp: Optional[str] = Field(default=None)
+    global_fields_retain: Optional[List[str]] = GLOBAL_RETAIN
+    global_fields_remove: Optional[List[str]] = GLOBAL_REMOVE
+    global_depth_start: Optional[int] = GLOBAL_DEPTH_START
+    global_depth_end: Optional[int] = GLOBAL_DEPTH_END
     _client: Optional[FigmaPy] = PrivateAttr()
 
     def _base_loader(
@@ -275,9 +227,9 @@ class FigmaApiWrapper(NonCodeIndexerToolkit):
             self._log_tool_event(f"Loading files: {file_keys_include}")
             for file_key in file_keys_include:
                 self._log_tool_event(f"Loading file `{file_key}`")
-                file = self._client.get_file(file_key)
+                file = self._client.get_file(file_key, geometry='depth=1') # fetch only top-level structure (only pages without inner components)
                 if not file:
-                    raise ToolException(f"Unexpected error while retrieving file {file_key}. Probably file is under editing. Try again later.")
+                    raise ToolException(f"Unexpected error while retrieving file {file_key}. Please try specifying the node-id of an inner page.")
                 metadata = {
                     'id': file_key,
                     'file_key': file_key,
@@ -332,20 +284,47 @@ class FigmaApiWrapper(NonCodeIndexerToolkit):
             for child in node['children']:
                 texts.extend(self.get_texts_recursive(child))
         return texts
+    
+    def _load_pages(self, document: Document):
+        file_key = document.metadata.get('id', '')
+        node_ids_include = document.metadata.pop('figma_pages_include', [])
+        node_ids_exclude = document.metadata.pop('figma_pages_exclude', [])
+        self._log_tool_event(f"Included pages: {node_ids_include}. Excluded pages: {node_ids_exclude}.")
+        if node_ids_include:
+            # try to fetch only specified pages/nodes in one request
+            file = self._get_file_nodes(file_key,','.join(node_ids_include)) # attempt to fetch only specified pages/nodes in one request
+            if file:
+                return [node['document'] for node in file.get('nodes', {}).values() if 'document' in node]
+        else:
+            # 
+            file = self._client.get_file(file_key)
+            if file:
+                figma_pages = file.document.get('children', [])
+                return [node for node in figma_pages if ('id' in node and node['id'].replace(':', '-') not in node_ids_exclude)]
+        # fallback to loading all pages and filtering them one by one
+        file = self._client.get_file(file_key, geometry='depth=1')
+        if not file:
+            raise ToolException(
+                f"Unexpected error while retrieving file {file_key}. Please try specifying the node-id of an inner page.")
+        figma_pages_raw = file.document.get('children', [])
+        # extract pages one by one
+        if node_ids_include:
+            return [self._get_file_nodes(file_key, node_id) for node_id in node_ids_include]
+        else:
+            # return [self._get_file_nodes(file_key, page["id"]) for page in figma_pages_raw if ('id' in page and page['id'].replace(':', '-') not in node_ids_exclude)]
+            result = []
+            for page in figma_pages_raw:
+                if 'id' in page and page['id'].replace(':', '-') not in node_ids_exclude:
+                    page_res = self._get_file_nodes(file_key, page["id"]).get('nodes', {}).get(page["id"], {}).get("document", {})
+                    result.append(page_res)
+            return result
 
     def _process_document(self, document: Document) -> Generator[Document, None, None]:
         file_key = document.metadata.get('id', '')
         self._log_tool_event(f"Loading details (images) for `{file_key}`")
-        figma_pages = self._client.get_file(file_key).document.get('children', [])
-        node_ids_include = document.metadata.pop('figma_pages_include', [])
-        node_ids_exclude = document.metadata.pop('figma_pages_exclude', [])
+        figma_pages = self._load_pages(document)
         node_types_include = [t.strip().lower() for t in document.metadata.pop('figma_nodes_include', [])]
         node_types_exclude = [t.strip().lower() for t in document.metadata.pop('figma_nodes_exclude', [])]
-        self._log_tool_event(f"Included pages: {node_ids_include}. Excluded pages: {node_ids_exclude}.")
-        if node_ids_include:
-            figma_pages = [node for node in figma_pages if ('id' in node and node['id'].replace(':', '-') in node_ids_include)]
-        elif node_ids_exclude:
-            figma_pages = [node for node in figma_pages if ('id' in node and node['id'].replace(':', '-') not in node_ids_exclude)]
 
         image_nodes = []
         text_nodes = {}
@@ -441,7 +420,7 @@ class FigmaApiWrapper(NonCodeIndexerToolkit):
                 default=None)),
             'node_types_include': (Optional[List[str]], Field(
                 description="List type of nodes to include in index: i.e. ['FRAME', 'COMPONENT', 'RECTANGLE', 'COMPONENT_SET', 'INSTANCE', 'VECTOR', ...].",
-                default=['frame'])),
+                default=None)),
             'node_types_exclude': (Optional[List[str]], Field(
                 description="List type of nodes to exclude from index. It is applied only if node_types_include is not provided: i.e. ['FRAME', 'COMPONENT', 'RECTANGLE', 'COMPONENT_SET', 'INSTANCE', 'VECTOR', ...]",
                 default=None))
@@ -546,6 +525,53 @@ class FigmaApiWrapper(NonCodeIndexerToolkit):
                 }
             return obj
 
+        def process_fields(obj, fields_retain=None, fields_remove=None, depth_start=1, depth_end=2, depth=1):
+            """
+            Reduces a nested dictionary or list by retaining or removing specified fields at certain depths.
+
+            - At each level, starting from `depth_start`, only fields in `fields_retain` are kept; fields in `fields_remove` are excluded unless also retained.
+            - Recursion stops at `depth_end`, ignoring all fields at or beyond this depth.
+            - Tracks which fields were retained and removed during processing.
+            - Returns a JSON string of the reduced object, plus lists of retained and removed fields.
+            """
+            fields_retain = set(fields_retain or [])
+            fields_remove = set(fields_remove or []) - fields_retain # fields in remove have lower priority than in retain
+
+            retained = set()
+            removed = set()
+
+            def _process(o, d):
+                if depth_end is not None and d >= depth_end:
+                    return None  # Ignore keys at or beyond cut_depth
+                if isinstance(o, dict):
+                    result = {}
+                    for k, v in o.items():
+                        if k in fields_remove:
+                            removed.add(k)
+                            continue
+                        if d >= depth_start:
+                            if k in fields_retain:
+                                retained.add(k)
+                                result[k] = _process(v, d + 1)  # process recursively
+                            else:
+                                # else: skip keys not in retain/default/to_process
+                                removed.add(k) # remember skipped keys
+                        else:
+                            # retained.add(k) # remember retained keys
+                            result[k] = _process(v, d + 1)
+                    return result
+                elif isinstance(o, list):
+                    return [_process(item, d + 1) for item in o]
+                else:
+                    return o
+
+            new_obj = _process(obj, depth)
+            return {
+                "result": json.dumps(new_obj),
+                "retained_fields": list(retained),
+                "removed_fields": list(removed)
+            }
+
         def fix_trailing_commas(json_string):
             json_string = re.sub(r",\s*,+", ",", json_string)
             json_string = re.sub(r",\s*([\]}])", r"\1", json_string)
@@ -555,10 +581,12 @@ class FigmaApiWrapper(NonCodeIndexerToolkit):
         @functools.wraps(func)
         def wrapper(self, *args, **kwargs):
             extra_params = kwargs.pop("extra_params", {})
-
             limit = extra_params.get("limit", self.global_limit)
             regexp = extra_params.get("regexp", self.global_regexp)
-
+            fields_retain = extra_params.get("fields_retain", self.global_fields_retain)
+            fields_remove = extra_params.get("fields_remove", self.global_fields_remove)
+            depth_start = extra_params.get("depth_start", self.global_depth_start)
+            depth_end = extra_params.get("depth_end", self.global_depth_end)
             try:
                 limit = int(limit)
                 result = func(self, *args, **kwargs)
@@ -568,13 +596,26 @@ class FigmaApiWrapper(NonCodeIndexerToolkit):
                     return ToolException(
                         "Response result is empty. Check your input parameters or credentials"
                     )
-
                 if isinstance(result, (dict, list)):
-                    processed_result = simplified_dict(result)
-                    result = json.dumps(processed_result)
+                    raw_result = result
+                    processed_result = simplified_dict(raw_result)
+                    raw_str_result = json.dumps(processed_result)
+                    str_result = raw_str_result
+                    if regexp:
+                        regexp = re.compile(regexp)
+                        str_result = re.sub(regexp, "", raw_str_result)
+                        str_result = fix_trailing_commas(str_result)
+                    if len(str_result) > limit:
+                        reduced = process_fields(raw_result, fields_retain=fields_retain, fields_remove=fields_remove, depth_start=depth_start, depth_end=depth_end)
+                        note = (f"Size of the output exceeds limit {limit}. Data reducing has been applied. "
+                                f"Starting from the depth_start = {depth_start} the following object fields were removed: {reduced['removed_fields']}. "
+                                f"The following fields were retained: {reduced['retained_fields']}. "
+                                f"Starting from depth_end = {depth_end} all fields were ignored. "
+                                f"You can adjust fields_retain, fields_remove, depth_start, depth_end, limit and regexp parameters to get more precise output")
+                        return f"## NOTE:\n{note}.\n## Result: {reduced['result']}"[:limit]
+                    return str_result
                 else:
                     result = json.dumps(result)
-
                 if regexp:
                     regexp = re.compile(regexp)
                     result = re.sub(regexp, "", result)
@@ -595,6 +636,12 @@ class FigmaApiWrapper(NonCodeIndexerToolkit):
             f"files/{file_key}/nodes?ids={str(ids)}", method="get"
         )
 
+    def _get_file_nodes(self, file_key: str, ids: str, **kwargs):
+        """Reads a specified file nodes by field key from Figma."""
+        return self._client.api_request(
+            f"files/{file_key}/nodes?ids={str(ids)}", method="get"
+        )
+
     @process_output
     def get_file(
         self,
