@@ -6,6 +6,7 @@ from pydantic import create_model, BaseModel, ConfigDict, Field
 from .api_wrapper import SQLApiWrapper
 from ..base.tool import BaseAction
 from .models import SQLDialect
+from ..elitea_base import filter_missconfigured_index_tools
 from ..utils import TOOLKIT_SPLITTER, clean_string, get_max_toolkit_length
 from ...configurations.sql import SqlConfiguration
 
@@ -46,6 +47,7 @@ class SQLToolkit(BaseToolkit):
         )
 
     @classmethod
+    @filter_missconfigured_index_tools
     def get_toolkit(cls, selected_tools: list[str] | None = None, toolkit_name: Optional[str] = None, **kwargs):
         if selected_tools is None:
             selected_tools = []
